@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2015, University of Amsterdam
+    Copyright (c)  2002-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -35,13 +35,11 @@
 
 #define O_DEBUG 1			/* provides time:time_debug(+Level) */
 //#define O_SAFE 1			/* extra safety checks */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
-#include <error.h>
+#include "error.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <signal.h>
@@ -53,12 +51,6 @@
 #include <errno.h>
 #include <assert.h>
 
-#ifdef SIGUSR2
-#define SIG_TIME SIGUSR2
-#else
-#define SIG_TIME SIGALRM
-#endif
-
 #ifdef O_SAFE
 #define __USE_GNU
 #endif
@@ -219,8 +211,9 @@ static pthread_t scheduler;		/* thread id of scheduler */
 static schedule the_schedule = {0};	/* the schedule */
 #define TheSchedule() (&the_schedule)	/* current schedule */
 
-int signal_function_set = FALSE;	/* signal function is set */
-static handler_t signal_function;	/* Current signal function */
+static int signal_function_set = FALSE;	/* signal function is set */
+static int sig_time = 0;
+static pl_sigaction_t saved_sigaction;	/* Old signal action */
 
 static int removeEvent(Event ev);
 
@@ -354,20 +347,29 @@ freeEvent(Event ev)
 
 
 static void
-cleanupHandler()
+cleanupHandler(void)
 { if ( signal_function_set )
-  { signal_function_set = FALSE;
-    PL_signal(SIG_TIME, signal_function);
+  { PL_sigaction(sig_time, &saved_sigaction, NULL);
+    signal_function_set = FALSE;
   }
 }
 
 
-static void
-installHandler()
+static int
+installHandler(void)
 { if ( !signal_function_set )
-  { signal_function = PL_signal(SIG_TIME|PL_SIGSYNC, on_alarm);
-    signal_function_set = TRUE;
+  { pl_sigaction_t act = {0};
+
+    act.sa_cfunction = on_alarm;
+    act.sa_flags     = PLSIG_SYNC;
+
+    if ( (sig_time = PL_sigaction(0, &act, &saved_sigaction)) > 0 )
+      signal_function_set = TRUE;
+    else
+      return PL_warning("Could not initialize alarm signal handler\n");
   }
+
+  return TRUE;
 }
 
 
@@ -524,11 +526,7 @@ alarm_loop(void * closure)
 			    (long)left.tv_sec,
 			    ev->pl_thread_id));
 	  set_bit(&signalled, ev->pl_thread_id);
-#ifdef __WINDOWS__
-	  PL_thread_raise(ev->pl_thread_id, SIG_TIME);
-#else
-	  pthread_kill(ev->thread_id, SIG_TIME);
-#endif
+	  PL_thread_raise(ev->pl_thread_id, sig_time);
 	}
       } else
 	break;
@@ -656,7 +654,6 @@ installEvent(Event ev)
 
     TheSchedule()->stop = FALSE;
     pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
     pthread_attr_setstacksize(&attr, 8192);
     rc = pthread_create(&scheduler, &attr, alarm_loop, NULL);
     pthread_attr_destroy(&attr);
@@ -1031,7 +1028,7 @@ current_alarms(term_t time, term_t goal, term_t id, term_t status,
 
 
 install_t
-install_time()
+install_time(void)
 { MODULE_user	  = PL_new_module(PL_new_atom("user"));
 
   FUNCTOR_alarm1  = PL_new_functor(PL_new_atom("$alarm"), 1);
@@ -1060,13 +1057,13 @@ install_time()
   PL_register_foreign("time_debug",	1, pl_time_debug,  0);
 #endif
 
-  installHandler();
-  PL_on_halt(cleanup, NULL);
+  if ( installHandler() )
+    PL_on_halt(cleanup, NULL);
   PL_thread_at_exit(cleanup_thread, NULL, TRUE);
 }
 
 
 install_t
-uninstall_time()
+uninstall_time(void)
 { cleanup(0, NULL);
 }
