@@ -89,10 +89,9 @@ typedef struct
 
 typedef struct _PL_thread_info_t
 { int		    pl_tid;		/* Prolog thread id */
-  size_t	    local_size;		/* Stack sizes */
-  size_t	    global_size;
-  size_t	    trail_size;
-  size_t	    stack_size;		/* system (C-) stack */
+  size_t	    stack_limit;	/* Stack sizes */
+  size_t	    table_space;	/* Max size for local tables */
+  size_t	    c_stack_size;	/* system (C-) stack */
   rc_cancel	    (*cancel)(int id);	/* cancel function */
   unsigned short    open_count;		/* for PL_thread_detach_engine() */
   unsigned	    detached      : 1;	/* detached thread */
@@ -164,8 +163,8 @@ typedef struct message_queue
   struct thread_message   *tail;	/* Tail of message queue */
   uint64_t	       sequence_next;	/* next for sequence id */
   word		       id;		/* Id of the queue */
-  long		       size;		/* # terms in queue */
-  long		       max_size;	/* Max # terms in queue */
+  size_t	       size;		/* # terms in queue */
+  size_t	       max_size;	/* Max # terms in queue */
   int		       waiting;		/* # waiting threads */
   int		       waiting_var;	/* # waiting with unbound */
   int		       wait_for_drain;	/* # threads waiting for write */
@@ -373,6 +372,7 @@ COMMON(Definition)	localiseDefinition(Definition def);
 COMMON(LocalDefinitions) new_ldef_vector(void);
 COMMON(void)		free_ldef_vector(LocalDefinitions ldefs);
 COMMON(void)		cleanupLocalDefinitions(PL_local_data_t *ld);
+COMMON(void)		destroyLocalDefinitions(Definition def);
 int			PL_mutex_lock(struct pl_mutex *m);
 int			PL_mutex_unlock(struct pl_mutex *m);
 int			PL_thread_raise(int tid, int sig);
@@ -449,7 +449,6 @@ COMMON(Atom**)	pl_atom_buckets_in_use(void);
 COMMON(Definition*)	predicates_in_use(void);
 COMMON(int)	pl_functor_table_in_use(FunctorTable functor_table);
 COMMON(int)	pl_kvs_in_use(KVS kvs);
-COMMON(void)	cgcActivatePredicate__LD(Definition def, gen_t gen ARG_LD);
 COMMON(gen_t)	pushPredicateAccess__LD(Definition def ARG_LD);
 COMMON(void)	popPredicateAccess__LD(Definition def ARG_LD);
 COMMON(size_t)	popNPredicateAccess__LD(size_t n ARG_LD);
